/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.model;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.Immutable;
import org.sonar.commons.MetricsRepository;

import javax.persistence.*;

@Immutable
@Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
@Table(name = "metrics")
@Entity(name = "Metric")
@NamedQueries({@NamedQuery(name = Metric.SQL_SELECT_BY_NAME, query = "SELECT m FROM Metric m WHERE m.name = :name"),
@NamedQuery(name = Metric.SQL_SELECT_ALL, query = "SELECT m FROM Metric m")
    })
public class Metric {

  public final static String SQL_SELECT_BY_NAME = "Metrics.selectByName";
  public static final String SQL_SELECT_ALL = "Metrics.selectAll";

  @Id
  @Column(name = "id")
  @SequenceGenerator(name = "METRICS_SEQ", sequenceName = "METRICS_SEQ")
  @GeneratedValue(strategy = GenerationType.AUTO, generator = "METRICS_SEQ")
  private Integer id;

  @Column(name = "name", updatable = false, nullable = false, length = 64)
  private String name;

  @Column(name = "description", updatable = false, nullable = true, length = 255)
  private String description;

  @Column(name = "value_type", updatable = false, nullable = false)
  @Enumerated(EnumType.ORDINAL)
  private MetricType type;

  @Column(name = "direction", updatable = false, nullable = true)
  private Integer direction;

  @Column(name = "domain", updatable = false, nullable = true, length = 60)
  private String domain;

  @Column(name = "short_name", updatable = false, nullable = true, length = 64)
  private String shortName;

  @Column(name = "long_name", updatable = false, nullable = true, length = 64)
  private String longName;

  @Column(name = "comparable", nullable = false)
  private Boolean comparable;

  public Metric() {
  }

  public Metric(String name) {
    this(name, MetricType.DOUBLE);
  }

  public Metric(Metrics metric, MetricType type) {
    this(metric.getName(), type);
  }

  public Metric(String name, MetricType type) {
    this(name, "", type, -1, null, name, name, Boolean.FALSE, Boolean.FALSE);
  }

  public Metric(String name, String description, MetricType type, Integer direction) {
    this.name = name;
    this.description = description;
    this.type = type;
    this.direction = direction;
    this.shortName = name;
    this.longName = name;
    this.comparable = Boolean.FALSE;
  }

  public Metric(String name, String description, MetricType type, Integer direction, String domain, String shortName, String longName, Boolean qualitative, Boolean comparable) {
    this.name = name;
    this.description = description;
    this.type = type;
    this.direction = direction;
    this.domain = domain;
    this.shortName = shortName;
    this.longName = longName;
    this.comparable = comparable;
  }

  public Integer getId() {
    return id;
  }

  public void setId(Integer id) {
    this.id = id;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public MetricType getType() {
    return type;
  }

  public void setType(MetricType type) {
    this.type = type;
  }

  public static Number getParsedValue(MetricType type, Double value) {
    if (value == null) {
      return null;
    }
    switch (type) {
      case DOUBLE:
        return value;
      case FLOAT:
      case PERCENTAGE:
        return value.floatValue();
      case INT:
        return value.intValue();
      case LONG:
        return value.longValue();
      case SHORT:
        return value.shortValue();
      case BYTE:
        return value.byteValue();
      case DURATION_MS:
        return value.longValue();
    }
    return null;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public boolean isMetric(Metrics metric) {
    if (name == null && metric.getName() != null) {
      return false;
    } else if (!name.equals(metric.getName()))
      return false;

    return true;
  }


  public Integer getDirection() {
    return direction;
  }

  public void setDirection(Integer direction) {
    this.direction = direction;
  }

  public String getDomain() {
    return domain;
  }

  public void setDomain(String domain) {
    this.domain = domain;
  }

  public String getShortName() {
    return shortName;
  }

  public void setShortName(String shortName) {
    this.shortName = shortName;
  }

  public String getLongName() {
    return longName;
  }

  public void setLongName(String longName) {
    this.longName = longName;
  }

  public Boolean isComparable() {
    return comparable;
  }

  public void setComparable(Boolean comparable) {
    this.comparable = comparable;
  }

  public int hashCode() {
    return name.hashCode();
  }

  public boolean equals(Object obj) {
    if (!(obj instanceof Metric)) {
      return false;
    }
    if (this == obj) {
      return true;
    }
    Metric other = (Metric) obj;
    return name.equals(other.getName());
  }

  public String toString() {
    return name;
  }

  public boolean isTendencyable(){
    if (MetricsRepository.NCSS.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.DUPLICATED_LINES.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.DUPLICATED_LINES_RATIO.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.SUREFIRE_TESTS.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.SUREFIRE_TIME.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.COMPLEXITY_TOTAL.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.CLASSES_COUNT.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.FUNCTIONS_COUNT.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.CODE_COVERAGE.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.ERROR_RULES_INDEX.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.ERROR_RULES_COUNT.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.WARNING_RULES_INDEX.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.WARNING_RULES_COUNT.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.FAILURE_RULES_INDEX.getName().equals(getName())){
      return true;
    }
    if (MetricsRepository.COMMENT_RATIO.getName().equals(getName())){
      return true;
    }
    return false;
  }
}
