/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.commons;

import ch.hortis.sonar.model.*;
import org.sonar.commons.database.DatabaseManager;

import javax.persistence.Query;
import java.util.*;

public class ReadOnlyRulesRepository implements RulesRepository {
  private Collection<Rule> rules;
  private Map<String, Rule> rulesByPluginKey;
  private Collection<RulesCategory> categories;
  private Map<Integer, RulesCategory> categoryById;
  private DatabaseManager databaseManager;
  private RulesProfile activeProfile;
  private Map<RulesCategory, Collection<Rule>> ruleErrorsActivatedByCategory;
  private Map<RulesCategory, Collection<Rule>> ruleWarningsActivatedByCategory;

  public ReadOnlyRulesRepository(DatabaseManager manager) {
    this.databaseManager = manager;
  }



  public Collection<Rule> getRules() {
    if (rules == null) {
      Query query = databaseManager.createQuery("SELECT r FROM Rule r");
      rules = query.getResultList();
    }
    return rules;
  }

  public Collection<Rule> getRulesByCategory(RulesCategory categ) {
    Collection<Rule> result = new ArrayList<Rule>();
    for (Rule rule : getRules()) {
      if (rule.getRulesCategory().equals(categ)) {
        result.add(rule);
      }
    }
    return result;
  }

  public Collection<RulesCategory> getRulesCategories() {
   if (categories ==null) {
     Query query = databaseManager.createQuery("SELECT r FROM RulesCategory r");
     categories = query.getResultList();
     categoryById = new HashMap<Integer, RulesCategory>();
     for (RulesCategory category : categories) {
       categoryById.put(category.getId(), category);
     }
   }
   return categories;
  }

  public RulesCategory getRulesCategory(Integer id) {
    getRulesCategories();
    return categoryById.get(id);
  }

  public Rule getRuleByPluginKey(String key) {
    if (rulesByPluginKey == null) {
      rulesByPluginKey = new HashMap<String, Rule>();
      Collection<Rule> rules = getRules();
      for (Rule rule : rules) {
        rulesByPluginKey.put(rule.getPluginRuleKey(), rule);
      }
    }
    return rulesByPluginKey.get(key);
  }

  public RulesProfile getActiveProfile() {
    if (activeProfile == null) {
      activeProfile = (RulesProfile) databaseManager.createQuery("select rp from RulesProfile rp where rp.active=true").getSingleResult();
    }
    return activeProfile;
  }

  public Collection<Rule> getRulesActivatedByCategoryAndByLevel(RulesCategory rulesCat, RuleFailureLevel level) {
    if (level == RuleFailureLevel.ERROR){
      return getErrorsRulesActivatedByCategory(rulesCat);
    } else {
      return getWarningsRulesActivatedByCategory(rulesCat);
    }
  }

  public List<RuleFailure> getRuleFailures(Snapshot snapshot) {
    Query query = databaseManager.createQuery(
      "SELECT rf FROM RuleFailure rf " +
        "WHERE rf.snapshotId=:snapshotId");
    query.setParameter("snapshotId", snapshot.getId());
    return (List<RuleFailure>) query.getResultList();
  }

  private Collection<Rule> getErrorsRulesActivatedByCategory(RulesCategory rulesCat) {
    if (ruleErrorsActivatedByCategory == null) {
      ruleErrorsActivatedByCategory = new HashMap<RulesCategory, Collection<Rule>>();
      for (RulesCategory rulesCategory : getRulesCategories()) {
        ruleErrorsActivatedByCategory.put(rulesCategory, getErrorRulesActivatedByCategoryAndByLevel(rulesCategory, RuleFailureLevel.ERROR));
      }
    }
    return ruleErrorsActivatedByCategory.get(rulesCat);
  }

  private Collection<Rule> getWarningsRulesActivatedByCategory(RulesCategory rulesCat) {
    if (ruleWarningsActivatedByCategory == null) {
      ruleWarningsActivatedByCategory = new HashMap<RulesCategory, Collection<Rule>>();
      for (RulesCategory rulesCategory : getRulesCategories()) {
        ruleWarningsActivatedByCategory.put(rulesCategory, getErrorRulesActivatedByCategoryAndByLevel(rulesCategory, RuleFailureLevel.WARNING));
      }
    }
    return ruleWarningsActivatedByCategory.get(rulesCat);
  }

  private Collection<Rule> getErrorRulesActivatedByCategoryAndByLevel(RulesCategory rulesCategory, RuleFailureLevel level) {
    Query query = databaseManager.createQuery(
      "SELECT r FROM Rule r, RulesCategory rc, ActiveRule ac " +
        "WHERE ac.rulesProfile = (:rules_profile) " +
        "AND ac.level = :level " +
        "AND r = ac.rule " +
        "AND r.rulesCategory = rc " +
        "AND rc = :rules_category ");
    query.setParameter("rules_profile", getActiveProfile());
    query.setParameter("rules_category", rulesCategory);
    query.setParameter("level", level);
    return (Collection<Rule>) query.getResultList();
  }
}
