/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.commons.database;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.dbcp.cpdsadapter.DriverAdapterCPDS;
import org.apache.commons.dbcp.datasources.SharedPoolDataSource;
import org.apache.commons.dbcp.datasources.SharedPoolDataSourceFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.naming.CompositeName;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.Name;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;
import javax.naming.RefAddr;
import javax.naming.Reference;
import javax.naming.Referenceable;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Properties;

public class JndiDatabaseConnector extends AbstractDatabaseConnector {
  private static final Logger LOG = LoggerFactory.getLogger(JndiDatabaseConnector.class);

  private EntityManagerFactory factory = null;
  private DataSource datasource = null;
  private boolean operational = false;

  public JndiDatabaseConnector(Configuration configuration) {
    super(configuration);
  }

  public void start() {
    if (!operational) {
      // get the datasource from JNDI
      datasource = loadDatasourceFromJndi();

      // bind the datasource to JNDI if it is not already done
      if (datasource == null) {
        createAndBindDatasource();
      }

      isOperational();
    }
  }

  public boolean isOperational() {
    if (!operational) {
      boolean goodVersion = checkSchemaVersion();
      if (goodVersion) {
        // initialize hibernate, which validates the database
        // The db should be OK because it's the good version. If not, this is a critical error
        getEntityManagerFactory();
        operational=true;
      }
    }
    return operational;
  }


  private String getJndiName() {
    return getConfiguration().getString(DatabaseProperties.PROP_JNDI_NAME, "jdbc/sonar");
  }

  private DataSource loadDatasourceFromJndi() {
    Context ctx = null;
    try {
      ctx = new InitialContext();
    } catch (NamingException e) {
      throw new JndiException("can not instantiate a JNDI context", e);
    }

    try {
      return (DataSource) ctx.lookup(getJndiName());

    } catch (NamingException e) {
      // datasource not found

    } finally {
      try {
        ctx.close();
      } catch (NamingException e) {
      }
    }
    return null;
  }

  private void createAndBindDatasource() {
    try {
      Configuration dsConfig = getConfiguration().subset("sonar.jdbc");
      Reference ref = new PropertiesReference(SharedPoolDataSource.class, dsConfig);
      SharedPoolDataSourceFactory factory = new SharedPoolDataSourceFactory();
      datasource = (SharedPoolDataSource) factory.getObjectInstance(ref, null, null, null);

      ref = new PropertiesReference(DriverAdapterCPDS.class, dsConfig);
      DriverAdapterCPDS cpds = new DriverAdapterCPDS();
      cpds = (DriverAdapterCPDS) cpds.getObjectInstance(ref, null, null, null);
      ((SharedPoolDataSource) datasource).setConnectionPoolDataSource(cpds);


    } catch (Exception e) {
      throw new RuntimeException("Cannot create the JDBC datasource", e);
    }

    // bind the datasource to JNDI
    Context ctx = null;
    try {
      ctx = new InitialContext();
      createJNDISubContexts(ctx, getJndiName());
      ctx.bind(getJndiName(), datasource);
      LOG.info("JDBC datasource bound to JNDI, name=" + getJndiName());

    } catch (NamingException e) {
      throw new JndiException("Can not bind JDBC datasource to JNDI", e);

    } finally {
      if (ctx != null) {
        try {
          ctx.close();
        } catch (NamingException e) {
        }
      }
    }
  }

  public void stop() {
    if (factory != null && factory.isOpen()) {
      factory.close();
    }

    factory = null;
    datasource = null;
    operational = false;
  }

  public Connection getConnection() throws SQLException {
    if (datasource != null) {
      return datasource.getConnection();
    }
    return null;
  }

  public EntityManager createEntityManager() throws WrongDatabaseVersionException {
    EntityManager manager = getEntityManagerFactory().createEntityManager();
    if (LOG_STATISTICS.isInfoEnabled()) {
      manager = new StatisticsEntityManager(manager, LOG_STATISTICS);
    }
    return manager;
  }

  public synchronized EntityManagerFactory getEntityManagerFactory() {
    if (factory != null) {
      return factory;
    }

    // other settings are stored into /META-INF/persistence.xml
    Properties props = new Properties();
    props.put("hibernate.connection.isolation", getConfiguration().getInt(DatabaseProperties.PROP_ISOLATION, Connection.TRANSACTION_READ_UNCOMMITTED));
    props.put("hibernate.hbm2ddl.auto", "validate");
    props.put("hibernate.connection.datasource", getJndiName());
    props.put("hibernate.dialect", getDialectClass(getDialect()));

    if (LOG_STATISTICS.isInfoEnabled()) {
      props.put("hibernate.generate_statistics", "true");
    }
    if (LOG_SQL.isInfoEnabled()) {
      props.put("hibernate.show_sql", "true");
    }

    if (getConfiguration().getBoolean(DatabaseProperties.PROP_HIBERNATE_SECOND_LEVEL_CACHE, true)) {
      props.put("hibernate.cache.provider_class", "org.hibernate.cache.EhCacheProvider");
      props.put("hibernate.cache.use_second_level_cache", "true");
      props.put("hibernate.cache.use_query_cache", "false");

    } else {
      props.put("hibernate.cache.provider_class", "org.hibernate.cache.NoCacheProvider");
      props.put("hibernate.cache.use_second_level_cache", "false");
      props.put("hibernate.cache.use_query_cache", "false");
    }

    factory = Persistence.createEntityManagerFactory("sonar", props);
    return factory;
  }

  private static class PropertiesReference extends Reference {
    private Configuration config;

    public PropertiesReference(Class<? extends Referenceable> referenceable, Configuration config) {
      super(referenceable.getName());
      this.config = config;
    }

    @Override
    public RefAddr get(String addrType) {
      LOG.debug("Commons DBCP requesting property " + addrType);
      final String propValue = config.getString(addrType);
      if (propValue != null) {
        return new RefAddr(addrType) {
          public Object getContent() {
            return propValue;
          }
        };
      }
      return null;
    }
  }

  private void createJNDISubContexts(Context ctx, String jndiBinding) throws NamingException {
    Name name = new CompositeName(jndiBinding);
    for ( int i = 0; i < name.size() -1; i++) {
      String namingContext = name.get(i);
      try {
        Object obj = ctx.lookup(namingContext);
        if ( !(obj instanceof Context) ) {
          throw new NamingException(namingContext + " is not a JNDI Context");
        }
      } catch (NameNotFoundException ex ) {
        ctx = ctx.createSubcontext(namingContext);
      }
    }
  }
}